'********************************************************
'*							*
'*		Propeller II ROM SHA-256/HMAC		*
'*							*
'*		Version 0.1				*
'*							*
'*		11/01/2012				*
'*							*
'********************************************************

'
' Usage:			commandlong := 0				'pre-clear command long
'				cognew($1CC, @commandlong)			'start SHA-256/HMAC in new cog
'
' Start here for HMAC:		commandlong := 1<<30 + (keysize-1)<<17 + @key	'start HMAC with key of keysize bytes (1..64)
'				repeat while commandlong			'(wait for command done)
'
' Start here for SHA-256:	commandlong := 2<<30 + (msgsize-1)<<17 + @msg	'hash msg of msgsize bytes (1..8192)
'				repeat while commandlong			'(wait for command done)
'
'				{issue more 2<<30 commands if msg > 8192 bytes}
'
'				commandlong := 3<<30 + @hashbuffer		'read resulting hash into hashbuffer (32 bytes)
'				repeat while commandlong			'(wait for command done)
'
'				{hasbuffer now contains result, ready for new 1<<30 or 2<<30 command}
'

DAT

		org

		setf	#%0_1111_0000	'configure movf for sbyte0 -> {dbyte3,dbyte2,dbyte1,dbyte0,dbyte3,...}

		call	#init_hash	'init hash, clear hmac mode, reset byte count
'
'
' Command Loop
'
command		rdlong	x,ptra		'wait for command (%cc_nnnnnnnnnnnnn_ppppppppppppppppp)
		tjz	x,#command

		setptrb	x		'get pointer (%ppppppppppppppppp)

		mov	count,x		'get count (%nnnnnnnnnnnnn)
		shl	count,#2
		shr	count,#2+17
		add	count,#1	'+1 for 1..8192 range

		shr	x,#32-2		'get command (%cc)

		cachex			'invalidate cache for fresh rdbytec's

		djz	x,#begin_hmac	'1 = begin hmac, pointer @key (count+1 bytes, 1..64)
		djz	x,#hash_bytes	'2 = hash bytes, pointer @message (count+1 bytes, 1..8192)
		djz	x,#read_hash	'3 = read hash, pointer @hashbuffer (32 bytes)

done		wrlong	zero,ptra	'clear command to signal done

		jmp	#command	'get next command
'
'
' Begin HMAC
'
begin_hmac	call	#end_hash	'end any hash in progress

:ipad		mov	x,#$00		'get and hash ipad key (full block)
		cmpr	count,bytes wc	'after key bytes, hash $00's to fill block
	if_c	rdbytec	x,ptrb++
		xor	x,#$36		'xor bytes with ipad ($36)
		call	#hash_byte	'(last iteration triggers hash_block, z=1)
	if_nz	jmp	#:ipad

		reps	#16,#2		'save opad key
		setinds	#opad_key,#w
		mov	indb,inda++
		xor	indb++,opad	'xor bytes with opad ($5C)

		mov	hmac,#1		'set hmac mode

		jmp	#done
'
'
' Hash Bytes
'
hash_bytes	rdbytec	x,ptrb++	'hash bytes
		call	#hash_byte
		djnz	count,#hash_bytes

		jmp	#done
'
'
' Read Hash
'
read_hash	tjz	hmac,#:not	'if not hmac, output hash


		call	#end_hash	'hmac, end current hash

		reps	#16,#1		'get opad key into w[0..15] (full block)
		setinds	#w,#opad_key
		mov	indb++,inda++

		call	#hash_block	'hash opad key

		reps	#8,#1		'get hashx[0..7] into w[0..7]
		setinds	#w,#hashx
		mov	indb++,inda++

		movd	hash_byte,#w+8	'account for opad key and hashx bytes
		mov	bytes,#64+32	'(1-1/2 blocks, 1/2 block needs end_hash)


:not		call	#end_hash	'end current hash

		setinda	#hashx		'store hashx[0..7] at pointer, big-endian
		mov	count,#8
:out		reps	#4,#2
		mov	x,inda++
		rol	x,#8
		wrbyte	x,ptrb++
		djnz	count,#:out

		jmp	#done
'
'
' End Hash - hash $80, any $00's needed to get to offset $38, then 8-byte length
'
end_hash	mov	length,bytes	'get message length in bits
		shl	length,#3

		mov	x,#$80		'hash end-of-message byte ($80)
:fill		call	#hash_byte	'(may trigger hash_block)
		mov	x,bytes		'hash any $00's needed to get to offset $38
		and	x,#$3F
		cmp	x,#$38	wz
		mov	x,#$00
	if_nz	jmp	#:fill

:len		test	bytes,#$04  wc	'hash 8-byte length, big-endian
	if_c	rol	length,#8	'(hash four $00's, then four length bytes)
	if_c	mov	x,length
		call	#hash_byte	'(last iteration triggers hash_block)
	if_nz	jmp	#:len

		reps	#8,#1		'save hash[0..7] into hashx[0..7]
		setinds	#hashx,#hash
		mov	indb++,inda++

init_hash	reps	#8,#1		'copy hash_init[0..7] into hash[0..7]
		setinds	#hash,#hash_init
		mov	indb++,inda++

		mov	hmac,#0		'clear hmac mode
		mov	bytes,#0	'reset byte count
init_hash_ret
end_hash_ret	ret
'
'
' Hash Byte - add byte to w[0..15] and hash block if full (z=1)
'
hash_byte	movf	w,x		'store byte into w[0..15], big-endian

		add	bytes,#1	'increment byte count

		test	bytes,#$03  wz	'every 4th byte, increment w pointer
	if_z	add	hash_byte,d0

		test	bytes,#$3F  wz	'every 64th byte, reset w pointer
	if_z	movd	hash_byte,#w

	if_z	call	#hash_block	'every 64th byte, hash block

hash_byte_ret	ret
'
'
' Hash Block - first extend w[0..15] into w[16..63] to generate schedule
'
hash_block	reps	#48,@:sch	'i = 16..63
		setinds	#w+16,#w+16-15+7'indb = @w[i], inda = @w[i-15+7]

		setinda	--7		's0 = (w[i-15] -> 7) ^ (w[i-15] -> 18) ^ (w[i-15] >> 3)
		mov	indb,inda--
		mov	x,indb
		rol	x,#18-7
		xor	x,indb
		ror	x,#18
		shr	indb,#3
		xor	indb,x

		add	indb,inda	'w[i] = s0 + w[i-16]

		setinda	++14		's1 = (w[i-2] -> 17) ^ (w[i-2] -> 19) ^ (w[i-2] >> 10)
		mov	x,inda
		mov	y,x
		rol	y,#19-17
		xor	y,x
		ror	y,#19
		shr	x,#10
		xor	x,y

		add	indb,x		'w[i] = s0 + w[i-16] + s1

		setinda	--5		'w[i] = s0 + w[i-16] + s1 + w[i-7]
:sch		add	indb++,inda


' Load variables from hash

		reps	#8,#1		'copy hash[0..7] into a..h
		setinds	#a,#hash
		mov	indb++,inda++


' Do 64 hash iterations on variables

		reps	#64,@:itr	'i = 0..63
		setinds	#k+0,#w+0	'indb = @k[i], inda = @w[i]

		mov	x,g		'ch = (e & f) ^ (!e & g)
		xor	x,f
		and	x,e
		xor	x,g

		mov	y,e		's1 = (e -> 6) ^ (e -> 11) ^ (e -> 25)
		rol	y,#11-6
		xor	y,e
		rol	y,#25-11
		xor	y,e
		ror	y,#25

		add	x,y		't1 = ch + s1
		add	x,indb++	't1 = ch + s1 + k[i]
		add	x,inda++	't1 = ch + s1 + k[i] + w[i]
		add	x,h		't1 = ch + s1 + k[i] + w[i] + h

		mov	y,c		'maj = (a & b) ^ (b & c) ^ (c & a)
		and	y,b
		or	y,a
		mov	h,c
		or	h,b
		and	y,h

		mov	h,a		's0 = (a -> 2) ^ (a -> 13) ^ (a -> 22)
		rol	h,#13-2
		xor	h,a
		rol	h,#22-13
		xor	h,a
		ror	h,#22

		add	y,h		't2 = maj + s0

		mov	h,g		'h = g
		mov	g,f		'g = f
		mov	f,e		'f = e
		mov	e,d		'e = d
		mov	d,c		'd = c
		mov	c,b		'c = b
		mov	b,a		'b = a

		add	e,x		'e = e + t1

		mov	a,x		'a = t1 + t2
:itr		add	a,y


' Add variables back into hash

		reps	#8,#1		'add a..h into hash[0..7]
		setinds	#hash,#a
		add	indb++,inda++

hash_block_ret	ret
'
'
' Defined data
'
zero		long	0
d0		long 	1 << 9

opad		long	$36363636 ^ $5C5C5C5C

hash_init	long	$6A09E667, $BB67AE85, $3C6EF372, $A54FF53A, $510E527F, $9B05688C, $1F83D9AB, $5BE0CD19	'fractionals of square roots of primes 2..19

k		long	$428A2F98, $71374491, $B5C0FBCF, $E9B5DBA5, $3956C25B, $59F111F1, $923F82A4, $AB1C5ED5	'fractionals of cube roots of primes 2..311
		long	$D807AA98, $12835B01, $243185BE, $550C7DC3, $72BE5D74, $80DEB1FE, $9BDC06A7, $C19BF174
		long	$E49B69C1, $EFBE4786, $0FC19DC6, $240CA1CC, $2DE92C6F, $4A7484AA, $5CB0A9DC, $76F988DA
		long	$983E5152, $A831C66D, $B00327C8, $BF597FC7, $C6E00BF3, $D5A79147, $06CA6351, $14292967
		long	$27B70A85, $2E1B2138, $4D2C6DFC, $53380D13, $650A7354, $766A0ABB, $81C2C92E, $92722C85
		long	$A2BFE8A1, $A81A664B, $C24B8B70, $C76C51A3, $D192E819, $D6990624, $F40E3585, $106AA070
		long	$19A4C116, $1E376C08, $2748774C, $34B0BCB5, $391C0CB3, $4ED8AA4A, $5B9CCA4F, $682E6FF3
		long	$748F82EE, $78A5636F, $84C87814, $8CC70208, $90BEFFFA, $A4506CEB, $BEF9A3F7, $C67178F2
'
'
' Undefined data
'
hmac		res	1
bytes		res	1
count		res	1
length		res	1

opad_key	res	16

hash		res	8
hashx		res	8

w		res	64

a		res	1
b		res	1
c		res	1
d		res	1
e		res	1
f		res	1
g		res	1
h		res	1

x		res	1
y		res	1
